@@ -15,11 +15,12 @@ use crate::{
     dumping::{self, Direction, Dumper},
     envelope::{Envelope, MessageKind},
     errors::{RequestError, SendError, TryRecvError, TrySendError},
+    mailbox::RecvResult,
     message::{Message, Request},
     messages,
     request_table::ResponseToken,
     routers::Singleton,
-    scope, trace_id,
+    scope,
 };
 
 pub(crate) use self::source::Source;
@@ -28,6 +29,7 @@ use self::{source::Combined, stats::Stats};
 mod source;
 mod stats;
 
+/// An actor execution context.
 pub struct Context<C = (), K = Singleton, S = ()> {
     book: AddressBook,
     dumper: Dumper,
@@ -49,35 +51,41 @@ enum Stage {
 }
 
 assert_impl_all!(Context: Send);
+// TODO: !Sync?
 
 impl<C, K, S> Context<C, K, S> {
+    /// Returns the actor's address.
     #[inline]
     pub fn addr(&self) -> Addr {
         self.addr
     }
 
+    /// Returns the current group's address.
     #[inline]
     pub fn group(&self) -> Addr {
         self.group
     }
 
     #[deprecated]
+    #[doc(hidden)]
     #[cfg(feature = "test-util")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "test-util")))]
     pub fn set_addr(&mut self, addr: Addr) {
         self.addr = addr;
     }
 
+    /// Returns the actual config.
     #[inline]
     pub fn config(&self) -> &C {
         &self.config
     }
 
+    /// Returns the actor's key.
     #[inline]
     pub fn key(&self) -> &K {
         &self.key
     }
 
+    /// Transforms the context to another one with the provided source.
     pub fn with<S1>(self, source: S1) -> Context<C, K, Combined<S, S1>> {
         Context {
             book: self.book,
@@ -93,6 +101,11 @@ impl<C, K, S> Context<C, K, S> {
         }
     }
 
+    /// Updates the actor's status.
+    ///
+    /// ```ignore
+    /// ctx.set_status(ActorStatus::ALARMING.with_details("something wrong"));
+    /// ```
     pub fn set_status(&self, status: ActorStatus) {
         let object = ward!(self.book.get_owned(self.addr));
         let actor = ward!(object.as_actor());
@@ -108,16 +121,66 @@ impl<C, K, S> Context<C, K, S> {
         ward!(object.as_actor(), return false).close()
     }
 
+    /// Sends a message using the routing system.
+    ///
+    /// Returns `Err` if the message hasn't reached any mailboxes.
+    ///
+    /// # Example
+    /// ```ignore
+    /// // Fire and forget.
+    /// let _ = ctx.send(SomethingHappened).await;
+    ///
+    /// // Fire or fail.
+    /// ctx.send(SomethingHappened).await?;
+    ///
+    /// // Fire or log.
+    /// if let Some(err) = ctx.send(SomethingHappened).await {
+    ///     warn!("...", error = err);
+    /// }
+    /// ```
     pub async fn send<M: Message>(&self, message: M) -> Result<(), SendError<M>> {
         let kind = MessageKind::Regular { sender: self.addr };
         self.do_send(message, kind).await
     }
 
+    /// Returns a request builder.
+    ///
+    /// # Example
+    /// ```ignore
+    /// // Request and wait for a response.
+    /// let response = ctx.request(SomeCommand).resolve().await?;
+    ///
+    /// // Request and wait for all responses.
+    /// for result in ctx.request(SomeCommand).all().resolve().await {
+    ///     // ...
+    /// }
+    /// ```
     #[inline]
     pub fn request<R: Request>(&self, request: R) -> RequestBuilder<'_, C, K, S, R, Any> {
         RequestBuilder::new(self, request)
     }
 
+    /// Returns a request builder to the specified recipient.
+    ///
+    /// # Example
+    /// ```ignore
+    /// // Request and wait for a response.
+    /// let response = ctx.request_to(addr, SomeCommand).resolve().await?;
+    ///
+    /// // Request and wait for all responses.
+    /// for result in ctx.request_to(addr, SomeCommand).all().resolve().await {
+    ///     // ...
+    /// }
+    /// ```
+    #[inline]
+    pub fn request_to<R: Request>(
+        &self,
+        recipient: Addr,
+        request: R,
+    ) -> RequestBuilder<'_, C, K, S, R, Any> {
+        RequestBuilder::new(self, request).to(recipient)
+    }
+
     async fn do_send<M: Message>(&self, message: M, kind: MessageKind) -> Result<(), SendError<M>> {
         self.stats.sent_messages_total::<M>();
 
@@ -170,6 +233,23 @@ impl<C, K, S> Context<C, K, S> {
         }
     }
 
+    /// Sends a message to the specified recipient.
+    ///
+    /// Returns `Err` if the message hasn't reached any mailboxes.
+    ///
+    /// # Example
+    /// ```ignore
+    /// // Fire and forget.
+    /// let _ = ctx.send_to(addr, SomethingHappened).await;
+    ///
+    /// // Fire or fail.
+    /// ctx.send_to(addr, SomethingHappened).await?;
+    ///
+    /// // Fire or log.
+    /// if let Some(err) = ctx.send_to(addr, SomethingHappened).await {
+    ///     warn!("...", error = err);
+    /// }
+    /// ```
     pub async fn send_to<M: Message>(
         &self,
         recipient: Addr,
@@ -200,6 +280,23 @@ impl<C, K, S> Context<C, K, S> {
         result.map_err(|err| SendError(err.0.do_downcast().into_message()))
     }
 
+    /// Tries to send a message to the specified recipient.
+    ///
+    /// Returns `Err` if the message hasn't reached mailboxes or they are full.
+    ///
+    /// # Example
+    /// ```ignore
+    /// // Fire and forget.
+    /// let _ = ctx.send(SomethingHappened).await;
+    ///
+    /// // Fire or fail.
+    /// ctx.send(SomethingHappened).await?;
+    ///
+    /// // Fire or log.
+    /// if let Some(err) = ctx.send(SomethingHappened).await {
+    ///     warn!("...", error = err);
+    /// }
+    /// ```
     pub fn try_send_to<M: Message>(
         &self,
         recipient: Addr,
@@ -228,6 +325,17 @@ impl<C, K, S> Context<C, K, S> {
         })
     }
 
+    /// Responds to the requester with the provided response.
+    ///
+    /// The token can be used only once.
+    ///
+    /// ```ignore
+    /// msg!(match envelope {
+    ///     (SomeRequest, token) => {
+    ///         ctx.respond(token, SomeResponse);
+    ///     }
+    /// })
+    /// ```
     pub fn respond<R: Request>(&self, token: ResponseToken<R>, message: R::Response) {
         if token.is_forgotten() {
             return;
@@ -256,88 +364,119 @@ impl<C, K, S> Context<C, K, S> {
     ///
     /// # Panics
     /// If the method is called again after returning `None`.
+    ///
+    /// # Examples
+    /// ```ignore
+    /// while let Some(envelope) = ctx.recv().await {
+    ///     msg!(match envelope {
+    ///         SomethingHappened => /* ... */,
+    ///     })
+    /// }
+    /// ```
     pub async fn recv(&mut self) -> Option<Envelope>
     where
         C: 'static,
         S: Source,
     {
-        if self.stage == Stage::Closed {
-            on_recv_after_close();
-        }
+        loop {
+            self.stats.message_handling_time_seconds();
 
-        self.stats.message_handling_time_seconds();
+            if self.stage == Stage::Closed {
+                on_recv_after_close();
+            }
 
-        // TODO: cache `OwnedEntry`?
-        let object = self.book.get_owned(self.addr)?;
-        let actor = object.as_actor()?;
+            // TODO: cache `OwnedEntry`?
+            let object = self.book.get_owned(self.addr)?;
+            let actor = object.as_actor()?;
 
-        if self.stage == Stage::PreRecv {
-            on_first_recv(&mut self.stage, actor);
-        }
+            if self.stage == Stage::PreRecv {
+                on_first_recv(&mut self.stage, actor);
+            }
 
-        let mailbox_fut = actor.recv();
-        pin_mut!(mailbox_fut);
+            let mailbox_fut = actor.recv();
+            pin_mut!(mailbox_fut);
 
-        let source_fut = poll_fn(|cx| self.source.poll_recv(cx));
-        pin_mut!(source_fut);
+            let source_fut = poll_fn(|cx| self.source.poll_recv(cx));
+            pin_mut!(source_fut);
 
-        let envelope = tokio::select! { biased;
-            envelope = mailbox_fut => envelope,
-            envelope = source_fut => envelope, // TODO: rerun select if `None`?
-            // Now no sources return `None`.
-        };
+            tokio::select! { biased;
+                result = mailbox_fut => match result {
+                    RecvResult::Data(envelope) => {
+                        if let Some(envelope) = self.post_recv(envelope) {
+                            return Some(envelope);
+                        }
+                    },
+                    RecvResult::Closed(trace_id) => {
+                        scope::set_trace_id(trace_id);
+                        on_input_closed(&mut self.stage, actor);
+                        return None;
+                    }
+                },
+                option = source_fut => {
+                    // Sources cannot return `None` for now.
+                    let envelope = option.expect("source cannot return None");
 
-        match envelope {
-            Some(envelope) => Some(self.post_recv(envelope)),
-            None => {
-                on_input_closed(&mut self.stage, actor);
-                // TODO: forward Terminate's trace_id.
-                scope::set_trace_id(trace_id::generate());
-                None
+                    if let Some(envelope) = self.post_recv(envelope) {
+                        return Some(envelope);
+                    }
+                },
             }
         }
     }
 
-    /// Receives the next envelope.
+    /// Receives the next envelope without waiting.
     ///
     /// # Panics
     /// If the method is called again after returning
     /// `Err(TryRecvError::Closed)`.
+    ///
+    /// # Examples
+    /// ```ignore
+    /// // Iterate over all available messages.
+    /// while let Ok(envelope) = ctx.try_recv() {
+    ///     msg!(match envelope {
+    ///         SomethingHappened => /* ... */,
+    ///     })
+    /// }
+    /// ```
     pub fn try_recv(&mut self) -> Result<Envelope, TryRecvError>
     where
         C: 'static,
     {
-        if self.stage == Stage::Closed {
-            on_recv_after_close();
-        }
+        loop {
+            self.stats.message_handling_time_seconds();
 
-        self.stats.message_handling_time_seconds();
-
-        let object = self.book.get(self.addr).ok_or(TryRecvError::Closed)?;
-        let actor = object.as_actor().ok_or(TryRecvError::Closed)?;
-
-        if self.stage == Stage::PreRecv {
-            on_first_recv(&mut self.stage, actor);
-        }
-
-        // TODO: poll the sources.
-        match actor.try_recv() {
-            Ok(envelope) => {
-                drop(object);
-                Ok(self.post_recv(envelope))
+            if self.stage == Stage::Closed {
+                on_recv_after_close();
             }
-            Err(err) => {
-                if err.is_closed() {
-                    on_input_closed(&mut self.stage, actor);
-                    // TODO: forward Terminate's trace_id.
-                    scope::set_trace_id(trace_id::generate());
+
+            let object = self.book.get(self.addr).ok_or(TryRecvError::Closed)?;
+            let actor = object.as_actor().ok_or(TryRecvError::Closed)?;
+
+            if self.stage == Stage::PreRecv {
+                on_first_recv(&mut self.stage, actor);
+            }
+
+            // TODO: poll the sources.
+            match actor.try_recv() {
+                Some(RecvResult::Data(envelope)) => {
+                    drop(object);
+
+                    if let Some(envelope) = self.post_recv(envelope) {
+                        return Ok(envelope);
+                    }
                 }
-                Err(err)
+                Some(RecvResult::Closed(trace_id)) => {
+                    scope::set_trace_id(trace_id);
+                    on_input_closed(&mut self.stage, actor);
+                    return Err(TryRecvError::Closed);
+                }
+                None => return Err(TryRecvError::Empty),
             }
         }
     }
 
-    fn post_recv(&mut self, envelope: Envelope) -> Envelope
+    fn post_recv(&mut self, envelope: Envelope) -> Option<Envelope>
     where
         C: 'static,
     {
@@ -353,16 +492,11 @@ impl<C, K, S> Context<C, K, S> {
                 self.respond(token, Ok(()));
                 envelope
             }
-            envelope => {
-                if envelope.is::<messages::Terminate>() {
-                    self.set_status(ActorStatus::TERMINATING);
-                }
-                envelope
-            }
+            envelope => envelope,
         });
 
         let message = envelope.message();
-        trace!("< {:?}", envelope);
+        trace!("< {:?}", message);
 
         if self.dumper.is_enabled() {
             self.dumper.dump(
@@ -375,17 +509,39 @@ impl<C, K, S> Context<C, K, S> {
             )
         }
 
+        // We should change the status after dumping the original message
+        // in order to see `ActorStatusReport` after that message.
+        if envelope.is::<messages::Terminate>() {
+            self.set_status(ActorStatus::TERMINATING);
+        }
+
         self.stats.message_waiting_time_seconds(&envelope);
-        envelope
+
+        msg!(match envelope {
+            (messages::Ping, token) => {
+                self.respond(token, ());
+                None
+            }
+            envelope => Some(envelope),
+        })
     }
 
     /// This is a part of private API for now.
+    /// We should provide a way to handle it asynchronous.
     #[doc(hidden)]
     pub async fn finished(&self, addr: Addr) {
         ward!(self.book.get_owned(addr)).finished().await;
     }
 
-    /// XXX: mb `BoundEnvelope<C>`?
+    /// Used to get the typed config from `ValidateConfig`.
+    /// ```ignore
+    /// msg!(match envelope {
+    ///     (ValidateConfig { config, .. }, token) => {
+    ///         let new_config = ctx.unpack_config(&config);
+    ///         ctx.respond(token, Err("oops".into()));
+    ///     }
+    /// })
+    /// ```
     pub fn unpack_config<'c>(&self, config: &'c AnyConfig) -> &'c C
     where
         C: for<'de> serde::Deserialize<'de> + 'static,
@@ -393,6 +549,9 @@ impl<C, K, S> Context<C, K, S> {
         config.get_user()
     }
 
+    /// Produces a new context that can be used for sending messages only.
+    ///
+    /// Pruned contexts are likely to be removed in favor of `Output`.
     pub fn pruned(&self) -> Context {
         Context {
             book: self.book.clone(),
@@ -518,7 +677,7 @@ impl<C, K: Clone> Clone for Context<C, K> {
 pub struct RequestBuilder<'c, C, K, S, R, M> {
     context: &'c Context<C, K, S>,
     request: R,
-    from: Option<Addr>,
+    to: Option<Addr>,
     marker: PhantomData<M>,
 }
 
@@ -531,7 +690,7 @@ impl<'c, C, K, S, R> RequestBuilder<'c, C, K, S, R, Any> {
         Self {
             context,
             request,
-            from: None,
+            to: None,
             marker: PhantomData,
         }
     }
@@ -541,7 +700,7 @@ impl<'c, C, K, S, R> RequestBuilder<'c, C, K, S, R, Any> {
         RequestBuilder {
             context: self.context,
             request: self.request,
-            from: self.from,
+            to: self.to,
             marker: PhantomData,
         }
     }
@@ -552,22 +711,31 @@ impl<'c, C, K, S, R> RequestBuilder<'c, C, K, S, R, Any> {
         RequestBuilder {
             context: self.context,
             request: self.request,
-            from: self.from,
+            to: self.to,
             marker: PhantomData,
         }
     }
 }
 
 impl<'c, C, K, S, R, M> RequestBuilder<'c, C, K, S, R, M> {
+    #[deprecated(note = "use `Context::request_to()` instead")]
+    #[doc(hidden)]
     #[inline]
-    pub fn from(mut self, addr: Addr) -> Self {
-        self.from = Some(addr);
+    pub fn from(self, addr: Addr) -> Self {
+        self.to(addr)
+    }
+
+    /// Specified the recipient of the request.
+    #[inline]
+    fn to(mut self, addr: Addr) -> Self {
+        self.to = Some(addr);
         self
     }
 }
 
 // TODO: add `pub async fn id() { ... }`
 impl<'c, C: 'static, K, S, R: Request> RequestBuilder<'c, C, S, K, R, Any> {
+    /// Waits for the response.
     pub async fn resolve(self) -> Result<R::Response, RequestError<R>> {
         // TODO: cache `OwnedEntry`?
         let this = self.context.addr;
@@ -579,7 +747,7 @@ impl<'c, C: 'static, K, S, R: Request> RequestBuilder<'c, C, S, K, R, Any> {
         let request_id = token.request_id;
         let kind = MessageKind::RequestAny(token);
 
-        let res = if let Some(recipient) = self.from {
+        let res = if let Some(recipient) = self.to {
             self.context.do_send_to(recipient, self.request, kind).await
         } else {
             self.context.do_send(self.request, kind).await
@@ -608,6 +776,7 @@ impl<'c, C: 'static, K, S, R: Request> RequestBuilder<'c, C, S, K, R, Any> {
 }
 
 impl<'c, C: 'static, K, S, R: Request> RequestBuilder<'c, C, K, S, R, All> {
+    /// Waits for the responses.
     pub async fn resolve(self) -> Vec<Result<R::Response, RequestError<R>>> {
         // TODO: cache `OwnedEntry`?
         let this = self.context.addr;
@@ -619,7 +788,7 @@ impl<'c, C: 'static, K, S, R: Request> RequestBuilder<'c, C, K, S, R, All> {
         let request_id = token.request_id;
         let kind = MessageKind::RequestAll(token);
 
-        let res = if let Some(recipient) = self.from {
+        let res = if let Some(recipient) = self.to {
             self.context.do_send_to(recipient, self.request, kind).await
         } else {
             self.context.do_send(self.request, kind).await
@@ -667,7 +836,7 @@ impl<'c, C: 'static, K, S, R: Request> RequestBuilder<'c, C, S, K, R, Forgotten>
         let token = ResponseToken::forgotten(self.context.book.clone());
         let kind = MessageKind::RequestAny(token);
 
-        let res = if let Some(recipient) = self.from {
+        let res = if let Some(recipient) = self.to {
             self.context.do_send_to(recipient, self.request, kind).await
         } else {
             self.context.do_send(self.request, kind).await
