@@ -14,7 +14,7 @@ pub use crate::{
     context::{Context, RequestBuilder},
     envelope::Envelope,
     group::{ActorGroup, Schema},
-    local::Local,
+    local::{Local, MoveOwnership},
     message::{Message, Request},
     request_table::ResponseToken,
     start::{start, try_start},
@@ -25,6 +25,7 @@ pub mod config;
 pub mod dumping;
 pub mod errors;
 pub mod group;
+pub mod logging;
 pub mod messages;
 pub mod node;
 pub mod routers;
