@@ -16,12 +16,23 @@ pub trait Router<C>: Send + Sync + 'static {
     fn route(&self, envelope: &Envelope) -> Outcome<Self::Key>;
 }
 
-#[derive(Debug)]
+/// Specifies which actors will get a message.
+#[derive(Debug)] // TODO(v0.2): mark as `non_exhaustive`.
 pub enum Outcome<T> {
+    /// Routes a message to an actor with the specified key.
+    /// If there is no active or restarting actor under this key,
+    /// such a actor will be started.
     Unicast(T),
-    Multicast(Vec<T>), // TODO: use `SmallVec`?
+    /// Routes a message to all actors with specified keys.
+    /// If there are no active or restarting actors under these keys,
+    /// corresponding actors will be started.
+    Multicast(Vec<T>), // TODO(v0.2): use `SmallVec`?
+    /// Routes a message to all active actors.
     Broadcast,
+    /// Discards a message.
+    /// If a message is discarded by everyone, the sending side gets an error.
     Discard,
+    /// Acts as `Broadcast` for system messages and `Discard` otherwise.
     Default,
 }
 
@@ -29,6 +40,7 @@ assert_eq_size!(Outcome<u64>, [u8; 32]);
 assert_eq_size!(Outcome<u128>, [u8; 32]);
 
 impl<T> Outcome<T> {
+    /// Transforms `Unicast` and `Multicast` variants.
     #[inline]
     pub fn map<U>(self, mut f: impl FnMut(T) -> U) -> Outcome<U> {
         match self {
@@ -40,6 +52,7 @@ impl<T> Outcome<T> {
         }
     }
 
+    /// Replaces the `Default` variant with the provided one.
     #[inline]
     pub fn or(self, outcome: Outcome<T>) -> Self {
         match self {
@@ -58,6 +71,7 @@ impl<C> Router<C> for () {
     }
 }
 
+/// A key used for actor groups containing only one actor.
 #[derive(Debug, Clone, Hash, PartialEq, Eq)]
 pub struct Singleton;
 
