@@ -5,24 +5,25 @@ use tokio::{
     pin, select,
     time::{sleep, timeout, Instant},
 };
-use tracing::{error, info, warn, Level};
+use tracing::{error, info, level_filters::LevelFilter, warn};
 
 use elfo_macros::message;
 
 use crate::{
     actor::{Actor, ActorMeta},
     addr::Addr,
+    config::SystemConfig,
     context::Context,
     demux::Demux,
     errors::{RequestError, StartError},
     message,
     messages::{Ping, Terminate, UpdateConfig},
     object::Object,
-    permissions::{AtomicPermissions, Permissions},
-    scope::Scope,
+    scope::{Scope, ScopeShared},
     signal::{Signal, SignalKind},
     subscription::SubscriptionManager,
     topology::Topology,
+    trace_id,
 };
 
 type Result<T, E = StartError> = std::result::Result<T, E>;
@@ -33,8 +34,7 @@ async fn start_entrypoints(ctx: &Context, topology: &Topology) -> Result<()> {
         .filter(|group| group.is_entrypoint)
         .map(|group| {
             let config = Default::default();
-            ctx.request(UpdateConfig { config })
-                .from(group.addr)
+            ctx.request_to(group.addr, UpdateConfig { config })
                 .resolve()
                 .or_else(|err| async move {
                     match err {
@@ -63,7 +63,7 @@ async fn wait_entrypoints(ctx: &Context, topology: &Topology) -> Result<()> {
     let futures = topology
         .actor_groups()
         .filter(|group| group.is_entrypoint)
-        .map(|group| ctx.request(Ping).from(group.addr).resolve());
+        .map(|group| ctx.request_to(group.addr, Ping).resolve());
 
     futures::future::try_join_all(futures)
         .await
@@ -122,14 +122,13 @@ pub async fn do_start<F: Future>(
         ),
     ));
 
-    let perm = Arc::new(AtomicPermissions::default());
-    perm.store({
-        let mut perm = Permissions::default();
-        perm.set_logging_enabled(Some(Level::INFO));
-        perm
-    });
+    let scope_shared = ScopeShared::new(Addr::NULL);
+    let mut config = SystemConfig::default();
+    config.logging.max_level = LevelFilter::INFO;
+    config.telemetry.per_actor_group = false;
+    scope_shared.configure(&config);
 
-    let scope = Scope::new(addr, Addr::NULL, meta, perm, Default::default());
+    let scope = Scope::new(trace_id::generate(), addr, meta, Arc::new(scope_shared));
     let f = async move {
         start_entrypoints(&ctx, &topology).await?;
         wait_entrypoints(&ctx, &topology).await?;
@@ -163,6 +162,7 @@ async fn termination(ctx: Context, topology: Topology) {
         select! {
             _ = &mut termination => return,
             Some(envelope) = ctx.recv() => {
+                // TODO: `Terminate::closing` on second `Ctrl-C`
                 // `Ctrl-C` has been pressed again.
                 // Terminate immediately.
                 if envelope.is::<TerminateSystem>() {
@@ -182,6 +182,7 @@ async fn do_termination(ctx: Context, topology: Topology) {
 }
 
 async fn terminate_groups(ctx: &Context, topology: &Topology, user: bool) {
+    // TODO: specify order of system groups.
     let futures = topology
         .actor_groups()
         .filter(|group| user ^ group.name.starts_with("system."))
