@@ -1,16 +1,19 @@
-use std::{any::Any, future::Future, mem, panic::AssertUnwindSafe, sync::Arc, time::Duration};
+use std::{
+    any::Any, future::Future, mem, ops::Deref, panic::AssertUnwindSafe, sync::Arc, time::Duration,
+};
 
 use dashmap::DashMap;
 use futures::{future::BoxFuture, FutureExt};
 use fxhash::FxBuildHasher;
 use metrics::{decrement_gauge, increment_gauge};
 use parking_lot::RwLock;
-use tracing::{error_span, info, warn, Instrument, Span};
+use tracing::{debug, error_span, info, warn, Instrument, Span};
 
-use crate as elfo;
+use crate::{self as elfo};
 use elfo_macros::msg_raw as msg;
-use elfo_utils::{CachePadded, ErrorChain, RateLimiter};
+use elfo_utils::CachePadded;
 
+use self::{backoff::Backoff, error_chain::ErrorChain, measure_poll::MeasurePoll};
 use crate::{
     actor::{Actor, ActorMeta, ActorStatus},
     addr::Addr,
@@ -22,25 +25,27 @@ use crate::{
     group::{RestartMode, RestartPolicy, TerminationPolicy},
     messages,
     object::{Object, ObjectArc},
-    permissions::AtomicPermissions,
     routers::{Outcome, Router},
-    scope::{self, Scope},
+    scope::{self, Scope, ScopeShared},
     subscription::SubscriptionManager,
+    trace_id,
 };
 
+mod backoff;
+mod error_chain;
+mod measure_poll;
+
 pub(crate) struct Supervisor<R: Router<C>, C, X> {
     meta: Arc<ActorMeta>,
     restart_policy: RestartPolicy,
     termination_policy: TerminationPolicy,
     span: Span,
     context: Context,
-    // TODO: replace with `crossbeam_utils::sync::ShardedLock`?
     objects: DashMap<R::Key, ObjectArc, FxBuildHasher>,
     router: R,
     exec: X,
     control: CachePadded<RwLock<ControlBlock<C>>>,
-    permissions: Arc<AtomicPermissions>,
-    logging_limiter: Arc<RateLimiter>,
+    scope_shared: Arc<ScopeShared>,
     status_subscription: Arc<SubscriptionManager>,
 }
 
@@ -59,7 +64,7 @@ macro_rules! get_or_spawn {
             None => $this
                 .objects
                 .entry(key.clone())
-                .or_try_insert_with(|| $this.spawn(key).ok_or(()))
+                .or_try_insert_with(|| $this.spawn(key, Default::default()).ok_or(()))
                 .map(|o| o.downgrade())
                 .ok(),
         }
@@ -97,26 +102,24 @@ where
             }),
             restart_policy,
             termination_policy,
-            context: ctx,
             objects: DashMap::default(),
             router,
             exec,
             control: CachePadded(RwLock::new(control)),
-            permissions: Default::default(),
-            logging_limiter: Default::default(),
+            scope_shared: Arc::new(ScopeShared::new(ctx.group())),
             status_subscription: Arc::new(status_subscription),
+            context: ctx,
         }
     }
 
     // This method shouldn't be called often.
     fn in_scope(&self, f: impl FnOnce()) {
-        Scope::with_trace_id(
+        Scope::new(
             scope::trace_id(),
             Addr::NULL,
-            self.context.group(),
             self.meta.clone(),
-            self.permissions.clone(),
-            Default::default(), // Do not limit logging in supervisor.
+            // TODO: do not limit logging in supervisor.
+            self.scope_shared.clone(),
         )
         .sync_within(|| self.span.in_scope(f));
     }
@@ -207,15 +210,18 @@ where
                 let outcome = self.router.route(&envelope);
                 self.do_handle(envelope, outcome.or(Outcome::Broadcast))
             }
+            messages::Ping => {
+                let outcome = self.router.route(&envelope);
+                self.do_handle(envelope, outcome.or(Outcome::Broadcast))
+            }
             _ => {
                 let outcome = self.router.route(&envelope);
-                self.do_handle(envelope, outcome)
+                self.do_handle(envelope, outcome.or(Outcome::Discard))
             }
         })
     }
 
     fn do_handle(self: &Arc<Self>, envelope: Envelope, outcome: Outcome<R::Key>) -> RouteReport {
-        // TODO: avoid copy & paste.
         match outcome {
             Outcome::Unicast(key) => {
                 let object = ward!(
@@ -229,79 +235,49 @@ where
                     Err(TrySendError::Closed(envelope)) => RouteReport::Closed(envelope),
                 }
             }
-            Outcome::Multicast(list) => {
-                let mut waiters = Vec::new();
-                let mut someone = false;
-
-                // TODO: avoid the loop in `try_send` case.
-                for key in list {
-                    let object = ward!(get_or_spawn!(self, key), continue);
-
-                    // TODO: we shouldn't clone `envelope` for the last object in a sequence.
-                    let envelope = ward!(
-                        envelope.duplicate(self.context.book()),
-                        continue // A requester has died, but `Multicast` is more insistent.
-                    );
-
-                    let actor = object.as_actor().expect("supervisor stores only actors");
-
-                    match actor.try_send(envelope) {
-                        Ok(_) => someone = true,
-                        Err(TrySendError::Full(envelope)) => {
-                            waiters.push((object.addr(), envelope))
-                        }
-                        Err(TrySendError::Closed(_)) => {}
-                    }
-                }
-
-                if waiters.is_empty() {
-                    if someone {
-                        RouteReport::Done
-                    } else {
-                        RouteReport::Closed(envelope)
-                    }
-                } else {
-                    RouteReport::WaitAll(someone, waiters)
-                }
-            }
-            Outcome::Broadcast => {
-                let mut waiters = Vec::new();
-                let mut someone = false;
-
-                // TODO: avoid the loop in `try_send` case.
-                for object in self.objects.iter() {
-                    // TODO: we shouldn't clone `envelope` for the last object in a sequence.
-                    let envelope = ward!(
-                        envelope.duplicate(self.context.book()),
-                        return RouteReport::Done // A requester has died.
-                    );
-
-                    let actor = object.as_actor().expect("supervisor stores only actors");
-
-                    match actor.try_send(envelope) {
-                        Ok(_) => someone = true,
-                        Err(TrySendError::Full(envelope)) => {
-                            waiters.push((object.addr(), envelope))
-                        }
-                        Err(TrySendError::Closed(_)) => {}
-                    }
-                }
-
-                if waiters.is_empty() {
-                    if someone {
-                        RouteReport::Done
-                    } else {
-                        RouteReport::Closed(envelope)
-                    }
-                } else {
-                    RouteReport::WaitAll(someone, waiters)
-                }
-            }
-            Outcome::Discard | Outcome::Default => RouteReport::Done,
+            Outcome::Multicast(list) => self.do_handle_multiple(
+                envelope,
+                list.into_iter().filter_map(|key| get_or_spawn!(self, key)),
+            ),
+            Outcome::Broadcast => self.do_handle_multiple(envelope, self.objects.iter()),
+            Outcome::Discard => RouteReport::Closed(envelope),
+            Outcome::Default => unreachable!("must be altered earlier"),
         }
     }
 
-    fn spawn(self: &Arc<Self>, key: R::Key) -> Option<ObjectArc> {
+    fn do_handle_multiple(
+        &self,
+        envelope: Envelope,
+        mut iter: impl Iterator<Item = impl Deref<Target = ObjectArc>>,
+    ) -> RouteReport {
+        let mut waiters = Vec::new();
+        let mut someone = false;
+
+        for object in &mut iter {
+            // TODO: we shouldn't clone `envelope` for the last object in a sequence.
+            // If a requester has died, go out.
+            let envelope = ward!(envelope.duplicate(self.context.book()), break);
+
+            let actor = object.as_actor().expect("supervisor stores only actors");
+            match actor.try_send(envelope) {
+                Ok(_) => someone = true,
+                Err(TrySendError::Full(envelope)) => waiters.push((object.addr(), envelope)),
+                Err(TrySendError::Closed(_)) => {}
+            }
+        }
+
+        if waiters.is_empty() {
+            if someone {
+                RouteReport::Done
+            } else {
+                RouteReport::Closed(envelope)
+            }
+        } else {
+            RouteReport::WaitAll(someone, waiters)
+        }
+    }
+
+    fn spawn(self: &Arc<Self>, key: R::Key, mut backoff: Backoff) -> Option<ObjectArc> {
         let control = self.control.read();
         if control.stop_spawning {
             return None;
@@ -368,17 +344,29 @@ where
                 .set_status(new_status);
 
             if need_to_restart {
-                increment_gauge!("elfo_restarting_actors", 1.);
-                // TODO: use `backoff`.
-                tokio::time::sleep(Duration::from_secs(5)).await;
-                decrement_gauge!("elfo_restarting_actors", 1.);
+                let after = backoff.next();
 
-                if let Some(object) = sv.spawn(key.clone()) {
+                if after == Duration::from_secs(0) {
+                    debug!("actor will be restarted immediately");
+                } else {
+                    debug!(?after, "actor will be restarted");
+
+                    increment_gauge!("elfo_restarting_actors", 1.);
+                    tokio::time::sleep(after).await;
+                    decrement_gauge!("elfo_restarting_actors", 1.);
+                }
+
+                // Restarted actors should have a new trace id.
+                scope::set_trace_id(trace_id::generate());
+
+                backoff.start();
+                if let Some(object) = sv.spawn(key.clone(), backoff) {
                     sv.objects.insert(key.clone(), object)
                 } else {
                     sv.objects.remove(&key).map(|(_, v)| v)
                 }
             } else {
+                debug!("actor won't be restarted");
                 sv.objects.remove(&key).map(|(_, v)| v)
             }
             .expect("where is the current actor?");
@@ -399,14 +387,9 @@ where
         );
         entry.insert(Object::new(addr, actor));
 
-        let scope = Scope::new(
-            addr,
-            self.context.group(),
-            meta,
-            self.permissions.clone(),
-            self.logging_limiter.clone(),
-        );
-        tokio::spawn(scope.within(fut.instrument(span)));
+        let scope = Scope::new(scope::trace_id(), addr, meta, self.scope_shared.clone());
+        let fut = MeasurePoll::new(fut.instrument(span));
+        tokio::spawn(scope.within(fut));
         let object = self.context.book().get_owned(addr).expect("just created");
         Some(object)
     }
@@ -431,14 +414,7 @@ where
         // Update the dumper's config.
         self.context.dumper().configure(&system.dumping);
 
-        // Update permissions.
-        let mut perm = self.permissions.load();
-        perm.set_logging_enabled(system.logging.max_level.to_tracing_level());
-        perm.set_telemetry_per_actor_group_enabled(system.telemetry.per_actor_group);
-        perm.set_telemetry_per_actor_key_enabled(system.telemetry.per_actor_key);
-        self.permissions.store(perm);
-
-        self.logging_limiter.configure(system.logging.max_rate);
+        self.scope_shared.configure(system);
 
         // Update user's config.
         control.config = Some(config.get_user::<C>().clone());
